/**
 * @author Sergey Burkov, http://www.wp3dprinting.com
 * @copyright 2017
 */

woo3dv.aabb = new Array();
woo3dv.resize_scale = 1;
woo3dv.default_scale = 100;
woo3dv.cookie_expire = parseInt(woo3dv.cookie_expire);
woo3dv.boundingBox=[];
woo3dv.initial_rotation_x = 0;
woo3dv.initial_rotation_y = 0;
woo3dv.initial_rotation_z = 0;


jQuery(document).ready(function(){

	if (!document.getElementById('woo3dv-cv')) return;

	jQuery("#rotation_x").bind('keyup mouseup', function () {
		woo3dvRotateModel('x', this.value);
		jQuery("#woo3dv_rotation_x").val(this.value);
	});
	jQuery("#rotation_y").bind('keyup mouseup', function () {
		woo3dvRotateModel('y', this.value);
		jQuery("#woo3dv_rotation_y").val(this.value);
	});
	jQuery("#rotation_z").bind('keyup mouseup', function () {
		woo3dvRotateModel('z', this.value);
		jQuery("#woo3dv_rotation_z").val(this.value);
	});

});



function woo3dvPreview() {


	//workaround for shortcode pages
	woo3dv.model_url = jQuery('#woo3dv_model_url').val();
	woo3dv.model_mtl = jQuery('#woo3dv_model_mtl').val();
	woo3dv.model_color = jQuery('#woo3dv_model_color').val();
	woo3dv.model_transparency = jQuery('#woo3dv_model_transparency').val();
	woo3dv.model_shininess = jQuery('#woo3dv_model_shininess').val();
	woo3dv.upload_url = jQuery('#woo3dv_upload_url').val();


	jQuery( function() {
		jQuery( "#woo3dv_dialog" ).dialog({
			modal: true,
			width: "600px"
		});
	} );

	window.woo3dv_canvas = document.getElementById('woo3dv-cv');
	woo3dvCanvasDetails();

	var logoTimerID = 0;

	woo3dv.targetRotation = 0;

	var model_type=woo3dv.model_url.split('.').pop().toLowerCase();
	woo3dvViewerInit(woo3dv.model_url, woo3dv.model_mtl, model_type, false);

	woo3dvAnimate();
}



function woo3dvViewerInit(model, mtl, ext) {
	var woo3dv_canvas = document.getElementById('woo3dv-cv');
	var woo3dv_canvas_width = jQuery('#woo3dv-cv').width()
	var woo3dv_canvas_height = jQuery('#woo3dv-cv').height()

	woo3dv.mtl=mtl;

	//3D Renderer
	woo3dv.renderer = Detector.webgl? new THREE.WebGLRenderer({ antialias: true, canvas: woo3dv_canvas, preserveDrawingBuffer: true }): new THREE.CanvasRenderer({canvas: woo3dv_canvas});
	woo3dv.renderer.setClearColor( parseInt(woo3dv.background1, 16) );
	woo3dv.renderer.setPixelRatio( window.devicePixelRatio );
	woo3dv.renderer.setSize( woo3dv_canvas_width, woo3dv_canvas_height );


	if (Detector.webgl) {

		woo3dv.renderer.gammaInput = true;
		woo3dv.renderer.gammaOutput = true;
		woo3dv.renderer.shadowMap.enabled = true;
		woo3dv.renderer.shadowMap.renderReverseSided = false;
		woo3dv.renderer.shadowMap.Type = THREE.PCFSoftShadowMap;
	}

	woo3dv.camera = new THREE.PerspectiveCamera( 35, woo3dv_canvas_width / woo3dv_canvas_height, 1, 1000 );
	woo3dv.camera.position.set( 0, 0, 0 );
	woo3dv.cameraTarget = new THREE.Vector3( 0, 0, 0 );

	woo3dv.scene = new THREE.Scene();
	//woo3dv.scene.fog = new THREE.Fog( 0x72645b, 1, 300 );

	//Group
	if (woo3dv.group) woo3dv.scene.remove(woo3dv.group);
	woo3dv.group = new THREE.Group();
	woo3dv.group.position.set( 0, 0, 0 )
	woo3dv.group.name = "group";
	woo3dv.scene.add( woo3dv.group );
	//Axis
	woo3dv.axis = new THREE.AxisHelper( 300 )
	woo3dv.scene.add( woo3dv.axis );


	//Light
	ambientLight = new THREE.AmbientLight(0x191919);
	woo3dv.scene.add(ambientLight);
	ambientLight.name = "light";



	directionalLight = new THREE.DirectionalLight( 0xffffff, 0.75 );
	directionalLight.name = "light";


	directionalLight2 = new THREE.DirectionalLight( 0xffffff, 0.75 );
	directionalLight2.name = "light2";


	woo3dv.controls = new THREE.OrbitControls( woo3dv.camera, woo3dv.renderer.domElement );
	if (woo3dv.auto_rotation=='on') {
		woo3dv.controls.autoRotate = true; 
		woo3dv.controls.autoRotateSpeed = 6;
	}

	woo3dv.controls.addEventListener( 'start', function() {
		//woo3dv.controls.autoRotate = false;
	});

	if (ext=='stl') {
		woo3dv.loader = new THREE.STLLoader();
	}
/*	else if (ext=='dae') {
		woo3dv.loader = new THREE.ColladaLoader();
	}*/
	else if (ext=='obj') {
		woo3dv.loader = new THREE.OBJLoader();
	}

	if (model.length>0) {

		var mtlLoader = new THREE.MTLLoader();
		mtlLoader.setPath( woo3dv.upload_url );

		if (ext=='obj' && mtl && mtl.length>0) {
			mtlLoader.load( mtl, function( materials ) {
				materials.preload();
				var objLoader = new THREE.OBJLoader();
				woo3dv.loader.setMaterials( materials );

				woo3dv.loader.load( model, function ( geometry ) {
		        	    woo3dvModelOnLoad(geometry);
				});
			});
		}
		else {
			woo3dv.loader.load( model, function ( geometry ) {
				woo3dvModelOnLoad(geometry)
			} );
		}
	}

	if (woo3dv.display_mode=='fullscreen') {
		woo3dvGoFullScreen();

	}


	window.addEventListener( 'resize', woo3dvOnWindowResize, false );

}


function woo3dvModelOnLoad(object) {
	woo3dv.object = object;
	geometry = object;
	if (object.type=='Group') {
		geometry = object.children[0].geometry;
		//todo: merge multiple geometries?
	}

	

	//Material
	var material = woo3dvCreateMaterial(woo3dv.shading);

	geometry.computeBoundingBox();
	woo3dv.boundingBox=geometry.boundingBox;
	if (object.type=='Group' && object.children.length>1) {
		var min_coords=[];
		var max_coords=[];
		for(var i=0;i<object.children.length;i++) {
			object.children[i].geometry.computeBoundingBox();
			if (i==0) {
				min_coords.x=object.children[i].geometry.boundingBox.min.x;
				min_coords.y=object.children[i].geometry.boundingBox.min.y;
				min_coords.z=object.children[i].geometry.boundingBox.min.z;
				max_coords.x=object.children[i].geometry.boundingBox.max.x;
				max_coords.y=object.children[i].geometry.boundingBox.max.y;
				max_coords.z=object.children[i].geometry.boundingBox.max.z;
			}
			else {
				if (object.children[i].geometry.boundingBox.min.x < min_coords.x) min_coords.x = object.children[i].geometry.boundingBox.min.x;
				if (object.children[i].geometry.boundingBox.min.y < min_coords.y) min_coords.y = object.children[i].geometry.boundingBox.min.y;
				if (object.children[i].geometry.boundingBox.min.z < min_coords.z) min_coords.z = object.children[i].geometry.boundingBox.min.z;

				if (object.children[i].geometry.boundingBox.max.x > max_coords.x) max_coords.x = object.children[i].geometry.boundingBox.max.x;
				if (object.children[i].geometry.boundingBox.max.y > max_coords.y) max_coords.y = object.children[i].geometry.boundingBox.max.y;
				if (object.children[i].geometry.boundingBox.max.z > max_coords.z) max_coords.z = object.children[i].geometry.boundingBox.max.z;
			}
		}
		woo3dv.boundingBox.min=min_coords;
		woo3dv.boundingBox.max=max_coords;
	}

	//Model


	woo3dvCreateModel(object, geometry, material, woo3dv.shading);

	if (object.type=='Group' && object.children.length>1)	{
	}
	else {
		geometry.center();
	}



	var mesh_width = geometry.boundingBox.max.x - geometry.boundingBox.min.x;
	var mesh_length = geometry.boundingBox.max.y - geometry.boundingBox.min.y;
	var mesh_height = geometry.boundingBox.max.z - geometry.boundingBox.min.z;

	var mesh_diagonal = Math.sqrt(mesh_width * mesh_width + mesh_length * mesh_length + mesh_height * mesh_height);

	if (Detector.webgl) {
		var canvas_width=woo3dv.renderer.getSize().width;
		var canvas_height=woo3dv.renderer.getSize().height;
	}
	else {
		var canvas_width=jQuery('#woo3dv-cv').width();
		var canvas_height=jQuery('#woo3dv-cv').height();
	}

	var canvas_diagonal = Math.sqrt(canvas_width * canvas_width + canvas_height * canvas_height);
	var model_dim = new Array();
	model_dim.x = woo3dv.boundingBox.max.x - woo3dv.boundingBox.min.x;
	model_dim.y = woo3dv.boundingBox.max.y - woo3dv.boundingBox.min.y;
	model_dim.z = woo3dv.boundingBox.max.z - woo3dv.boundingBox.min.z;

	var max_side = Math.max(model_dim.x, model_dim.y, model_dim.z)*1.2
	var axis_length = Math.max(mesh_width, mesh_length);
	var axis_width = Math.min(mesh_width, mesh_length);
	woo3dv.spritey_x = woo3dvMakeTextSprite( " X ", 
		{ fontsize: 40, borderColor: {r:255, g:102, b:0, a:1.0}, backgroundColor: {r:255, g:255, b:255, a:0.8} } );
	woo3dv.spritey_x.position.set((mesh_width/2)+10,0,0);
	woo3dv.spritey_x.rotation.z = 90 * Math.PI/180;
	woo3dv.spritey_x.rotation.x = -90 * Math.PI/180;
	woo3dv.scene.add( woo3dv.spritey_x );
	woo3dv.spritey_y = woo3dvMakeTextSprite( " Y ", 
		{ fontsize: 40, borderColor: {r:51, g:51, b:255, a:1.0}, backgroundColor: {r:255, g:255, b:255, a:0.8} } );
	woo3dv.spritey_y.position.set(0,0,(mesh_length/2)+10);
	woo3dv.spritey_y.rotation.z = 90 * Math.PI/180;
	woo3dv.spritey_y.rotation.x = -90 * Math.PI/180;
	woo3dv.scene.add( woo3dv.spritey_y );
	woo3dv.spritey_z = woo3dvMakeTextSprite( " Z ", 
		{ fontsize: 40, borderColor: {r:51, g:204, b:51, a:1.0}, backgroundColor: {r:255, g:255, b:255, a:0.8} } );
	woo3dv.spritey_z.position.set(0,(mesh_height/2)+10,0);
	woo3dv.spritey_z.rotation.z = 90 * Math.PI/180;
	woo3dv.spritey_z.rotation.x = -90 * Math.PI/180;
	woo3dv.scene.add( woo3dv.spritey_z );

	//Camera
	woo3dv.camera.position.set(max_side*woo3dv.resize_scale, max_side*woo3dv.resize_scale, max_side*woo3dv.resize_scale);

	woo3dv.camera.far=10000;
	woo3dv.camera.updateProjectionMatrix();

	//Ground
	if (Detector.webgl) {
		if (woo3dv.ground_mirror=='on') {
			var plane_shininess = 2500;
			var plane_transparent = true;
			var plane_opacity = 0.6;
		}
		else {
			var plane_shininess = 30;
			var plane_transparent = false;
			var plane_opacity = 1;
		}
		plane = new THREE.Mesh(
			new THREE.PlaneBufferGeometry( 2000, 2000 ),
			new THREE.MeshPhongMaterial ( { color: parseInt(woo3dv.ground_color, 16), transparent:plane_transparent, opacity:plane_opacity, shininess: plane_shininess } ) 
		);
		plane.rotation.x = -Math.PI/2;
		plane.position.y = woo3dv.boundingBox.min.z;
		plane.receiveShadow = true;
		plane.castShadow = true;
		plane.name = 'ground';
		woo3dv.scene.add( plane );
		if (woo3dv.ground_mirror=='on') {
			var planeGeo = new THREE.PlaneBufferGeometry( 2000, 2000 );
			woo3dv.groundMirror = new THREE.Mirror( woo3dv.renderer, woo3dv.camera, { clipBias: 0.003, textureWidth: canvas_width, textureHeight: canvas_height, color: 0xaaaaaa } );
			var mirrorMesh = new THREE.Mesh( planeGeo, woo3dv.groundMirror.material );
			mirrorMesh.position.y = woo3dv.boundingBox.min.z-0.1;
			mirrorMesh.add( woo3dv.groundMirror );
			mirrorMesh.rotateX( - Math.PI / 2 );
			woo3dv.scene.add( mirrorMesh );
		}

	}

	//Grid
	if (woo3dv.show_grid=='on' && woo3dv.plane_color.length>0) {

		var size = 1000, step = 50;
		var grid_geometry = new THREE.Geometry();
		for ( var i = - size; i <= size; i += step ) {
			grid_geometry.vertices.push( new THREE.Vector3( - size, woo3dv.boundingBox.min.z, i ) );
			grid_geometry.vertices.push( new THREE.Vector3(   size, woo3dv.boundingBox.min.z, i ) );
			grid_geometry.vertices.push( new THREE.Vector3( i, woo3dv.boundingBox.min.z, - size ) );
			grid_geometry.vertices.push( new THREE.Vector3( i, woo3dv.boundingBox.min.z,   size ) );
		
		}


		var grid_material = new THREE.LineBasicMaterial( { color: parseInt(woo3dv.plane_color, 16), opacity: 0.2 } );
		var line = new THREE.LineSegments( grid_geometry, grid_material );
		line.name = "grid";
		woo3dv.scene.add( line );
		woo3dv.group.add( line );
	}

	
	directionalLight.position.set( max_side*2, max_side*2, max_side*2 );
	directionalLight2.position.set( -max_side*2, max_side*2, -max_side*2 );
	if (Detector.webgl && woo3dv.show_shadow=='on') {
		directionalLight.castShadow = true;
		directionalLight2.castShadow = true;
		woo3dvMakeShadow();
	}
	woo3dv.scene.add( directionalLight );
	woo3dv.scene.add( directionalLight2 );


	woo3dvDisplayUserDefinedProgressBar(false);

	if (jQuery("#woo3dv_rotation_x").val()!=0)
		woo3dvRotateModel('x', jQuery("#rotation_x").val());
	if (jQuery("#woo3dv_rotation_y").val()!=0)
		woo3dvRotateModel('y', jQuery("#rotation_y").val());
	if (jQuery("#woo3dv_rotation_z").val()!=0)
		woo3dvRotateModel('z', jQuery("#rotation_z").val());




}
function woo3dvMakeTextSprite( message, parameters )  {
        if ( parameters === undefined ) parameters = {};
        var fontface = parameters.hasOwnProperty("fontface") ? parameters["fontface"] : "Arial";
        var fontsize = parameters.hasOwnProperty("fontsize") ? parameters["fontsize"] : 18;
        var borderThickness = parameters.hasOwnProperty("borderThickness") ? parameters["borderThickness"] : 4;
        var borderColor = parameters.hasOwnProperty("borderColor") ?parameters["borderColor"] : { r:0, g:0, b:0, a:1.0 };
        var backgroundColor = parameters.hasOwnProperty("backgroundColor") ?parameters["backgroundColor"] : { r:255, g:255, b:255, a:1.0 };
        var textColor = parameters.hasOwnProperty("textColor") ?parameters["textColor"] : { r:0, g:0, b:0, a:1.0 };
        var canvas = document.createElement('canvas');
        var context = canvas.getContext('2d');
        context.font = "Bold " + fontsize + "px " + fontface;
        var metrics = context.measureText( message );
        var textWidth = metrics.width;
        context.fillStyle   = "rgba(" + backgroundColor.r + "," + backgroundColor.g + "," + backgroundColor.b + "," + backgroundColor.a + ")";
        context.strokeStyle = "rgba(" + borderColor.r + "," + borderColor.g + "," + borderColor.b + "," + borderColor.a + ")";
        context.lineWidth = borderThickness;
        woo3dvRoundRect(context, borderThickness/2, borderThickness/2, (textWidth + borderThickness) * 1.1, fontsize * 1.4 + borderThickness, 8);
        context.fillStyle = "rgba("+textColor.r+", "+textColor.g+", "+textColor.b+", 1.0)";
        context.fillText( message, borderThickness, fontsize + borderThickness);
        var texture = new THREE.Texture(canvas) 
        texture.needsUpdate = true;
        var spriteMaterial = new THREE.SpriteMaterial( { map: texture, useScreenCoordinates: false } );
        var sprite = new THREE.Sprite( spriteMaterial );
        sprite.scale.set(0.5 * fontsize, 0.25 * fontsize, 0.75 * fontsize);
        return sprite;  
}
function woo3dvRoundRect(ctx, x, y, w, h, r) {
    ctx.beginPath();
    ctx.moveTo(x+r, y);
    ctx.lineTo(x+w-r, y);
    ctx.quadraticCurveTo(x+w, y, x+w, y+r);
    ctx.lineTo(x+w, y+h-r);
    ctx.quadraticCurveTo(x+w, y+h, x+w-r, y+h);
    ctx.lineTo(x+r, y+h);
    ctx.quadraticCurveTo(x, y+h, x, y+h-r);
    ctx.lineTo(x, y+r);
    ctx.quadraticCurveTo(x, y, x+r, y);
    ctx.closePath();
    ctx.fill();
    ctx.stroke();   
}

function woo3dvCreateMaterial(model_shading) {

	var color = new THREE.Color( woo3dv.model_color );
	var shininess = woo3dvGetCurrentShininess();
	var transparency = woo3dvGetCurrentTransparency();

	color.offsetHSL(0, 0, -0.1);

	if (Detector.webgl) {
		if (model_shading=='smooth') {
			var shading = THREE.SmoothShading;
		}
		else {
			var shading = THREE.FlatShading;
		}

		var material = new THREE.MeshPhongMaterial( { color: color, specular: shininess.specular, shininess: shininess.shininess, transparent:true, opacity:transparency, wireframe:false, shading:shading } );
	}
	else {

		var material = new THREE.MeshLambertMaterial( { color: color, vertexColors: THREE.FaceColors, wireframe: false, overdraw:1, shading:THREE.FlatShading } );
	}
	return material;
}

function woo3dvCreateModel(object, geometry, material, shading) {

	var attrib = geometry.getAttribute('position');
	if(attrib === undefined) {
		throw new Error('a given BufferGeometry object must have a position attribute.');
	}
	var positions = attrib.array;
	var vertices = [];
	for(var i = 0, n = positions.length; i < n; i += 3) {
		var x = positions[i];
		var y = positions[i + 1];
		var z = positions[i + 2];
		vertices.push(new THREE.Vector3(x, y, z));
	}
	var faces = [];
	for(var i = 0, n = vertices.length; i < n; i += 3) {
		faces.push(new THREE.Face3(i, i + 1, i + 2));
	}

	var new_geometry = new THREE.Geometry();
	new_geometry.vertices = vertices;
	new_geometry.faces = faces;
	new_geometry.computeFaceNormals();              
	new_geometry.computeVertexNormals();
	new_geometry.computeBoundingBox();

	geometry = new_geometry;
	geometry.center();

	if (shading=='smooth' && Detector.webgl) {
                var smooth_geometry = new THREE.Geometry();
                smooth_geometry.vertices = vertices;
                smooth_geometry.faces = faces;
                smooth_geometry.computeFaceNormals();              
                smooth_geometry.mergeVertices();
                smooth_geometry.computeVertexNormals();
		smooth_geometry.computeBoundingBox();
		geometry = smooth_geometry;
                woo3dv.model_mesh = new THREE.Mesh(geometry, material);
	}


	else {
		woo3dv.model_mesh = new THREE.Mesh( geometry, material );

	}

	if (woo3dv.object.type=='Group') {
		if (!woo3dv.mtl || woo3dv.mtl.length==0) {
			//woo3dv.object.children[0].material=woo3dv.model_mesh.material;
			for (var i=0;i<woo3dv.object.children.length;i++) {
				woo3dv.object.children[i].material=woo3dv.model_mesh.material;
			}
		}

		woo3dv.object.position.set( 0, 0, 0 );
		woo3dv.object.rotation.z = 90 * Math.PI/180;
		woo3dv.object.rotation.x = -90 * Math.PI/180;
		woo3dv.object.name = "object";
		woo3dv.initial_rotation_x = woo3dv.object.rotation.x;
		woo3dv.initial_rotation_y = woo3dv.object.rotation.y;
		woo3dv.initial_rotation_z = woo3dv.object.rotation.z;
		if (Detector.webgl) {
			for (var i=0;i<woo3dv.object.children.length;i++) {
				woo3dv.object.children[i].castShadow = true;
				woo3dv.object.children[i].receiveShadow = true;
			}
		}
		woo3dv.scene.add( woo3dv.object );
		woo3dv.group.add( woo3dv.object );
	}
	else {
		woo3dv.model_mesh.position.set( 0, 0, 0 );
		woo3dv.model_mesh.rotation.z = 90 * Math.PI/180;
		woo3dv.model_mesh.rotation.x = -90 * Math.PI/180;

		woo3dv.initial_rotation_x = woo3dv.model_mesh.rotation.x;
		woo3dv.initial_rotation_y = woo3dv.model_mesh.rotation.y;
		woo3dv.initial_rotation_z = woo3dv.model_mesh.rotation.z;

		woo3dv.model_mesh.name = "model";
		if (Detector.webgl) {
			woo3dv.model_mesh.castShadow = true;
			woo3dv.model_mesh.receiveShadow = true;
		}
		woo3dv.scene.add( woo3dv.model_mesh );
		woo3dv.group.add( woo3dv.model_mesh );
	}




}

function woo3dvMakeShadow() {
	var model_dim = new Array();
	model_dim.x = woo3dv.boundingBox.max.x - woo3dv.boundingBox.min.x;
	model_dim.y = woo3dv.boundingBox.max.y - woo3dv.boundingBox.min.y;
	model_dim.z = woo3dv.boundingBox.max.z - woo3dv.boundingBox.min.z;

	var max_side = Math.max(model_dim.x, model_dim.y, model_dim.z)
  	var bias = -0.001;
	var d = max_side*woo3dv.resize_scale;
	if (d<30) bias = -0.0001;
	directionalLight2.shadow.camera.left = directionalLight.shadow.camera.left = -d;
	directionalLight2.shadow.camera.right = directionalLight.shadow.camera.right = d;
	directionalLight2.shadow.camera.top = directionalLight.shadow.camera.top = d;
	directionalLight2.shadow.camera.bottom = directionalLight.shadow.camera.bottom = -d;
	directionalLight2.shadow.camera.near = directionalLight.shadow.camera.near = 1;
	directionalLight2.shadow.camera.far = directionalLight.shadow.camera.far = woo3dv.camera.far;
	directionalLight2.shadow.mapSize.width = directionalLight.shadow.mapSize.width = 2048;
	directionalLight2.shadow.mapSize.height = directionalLight.shadow.mapSize.height = 2048;
	directionalLight2.shadow.bias = directionalLight.shadow.bias = bias;

	if (directionalLight.shadow.map) {
		directionalLight.shadow.map.dispose(); 
		directionalLight.shadow.map = null;
		directionalLight2.shadow.map.dispose(); 
		directionalLight2.shadow.map = null;
	}
}



function woo3dvOnWindowResize() {

	var woo3dv_canvas_width = jQuery('div.images').width()
	var woo3dv_canvas_height = jQuery('#woo3dv-viewer').width()
	woo3dv.camera.aspect = woo3dv_canvas_width / woo3dv_canvas_height;
	woo3dv.camera.updateProjectionMatrix();
	woo3dv.renderer.setSize( woo3dv_canvas_width, woo3dv_canvas_height );


	woo3dvCanvasDetails();
}

function woo3dvCanvasDetails() {
	jQuery("#woo3dv-file-loading").css({
		top: jQuery("#woo3dv-cv").position().top+jQuery("#woo3dv-cv").height()/2-jQuery("#woo3dv-file-loading").height()/2,
		left: jQuery("#woo3dv-cv").position().left + jQuery("#woo3dv-cv").width()/2-jQuery("#woo3dv-file-loading").width()/2
	}) ;
}


woo3dv.frame_count=0;
function woo3dvAnimate() {
	window.requestAnimationFrame( woo3dvAnimate );
	woo3dv.group.rotation.y += ( woo3dv.targetRotation - woo3dv.group.rotation.y ) * 0.05;
	woo3dv.controls.update();

	woo3dvRender();

}


function woo3dvRender() {

	if (Detector.webgl && woo3dv.ground_mirror=='on' && typeof(woo3dv.groundMirror)!=='undefined')
		woo3dv.groundMirror.render();
	woo3dv.renderer.render( woo3dv.scene, woo3dv.camera );



		








}






















function woo3dvChangeModelColor(model_color) {

	if (!woo3dv.model_mesh) return;

	woo3dv.model_mesh.material.color.set(model_color);
	woo3dv.model_mesh.material.color.offsetHSL(0, 0, -0.1);
	if (Detector.webgl) {
		var model_shininess = woo3dvGetCurrentShininess();
		woo3dv.model_mesh.material.shininess = model_shininess.shininess;
		woo3dv.model_mesh.material.specular.set(model_shininess.specular);

		var model_transparency = woo3dvGetCurrentTransparency();
		woo3dv.model_mesh.material.opacity = model_transparency;


		if (woo3dv.object && woo3dv.object.type=='Group' && !(woo3dv.mtl && woo3dv.mtl.length>0)) {
			for (var i=0;i<woo3dv.object.children.length;i++) {
				woo3dv.object.children[i].material=woo3dv.model_mesh.material;

			}

		}
	}

	jQuery('#woo3dv_model_color').val(model_color)
};

function woo3dvGetCurrentColor() {
	var model_color = '#000000';
	if (typeof(jQuery('input[name=product_coating]:checked').closest('li').data('color'))!=='undefined' && jQuery('input[name=product_coating]:checked').closest('li').data('color').length>0 )
		model_color = jQuery('input[name=product_coating]:checked').closest('li').data('color');
	else
		model_color = jQuery('input[name=product_filament]:checked').closest('li').data('color');
	return model_color;

}

function woo3dvGetCurrentShininess() {

	switch(woo3dv.model_shininess) {
		case 'plastic':
			var shininess = 150;
			var specular = 0x111111;
	        break;
		case 'wood':
			var shininess = 15;
			var specular = 0x111111;
	        break;
		case 'metal':
			var shininess = 500;
			var specular = 0xc9c9c9;
	        break;
		default:
			var shininess = 150;
			var specular = 0x111111;

	}

	return {shininess: shininess, specular: specular};
}

function woo3dvSetCurrentShininess(model_shininess) {
	switch(model_shininess) {
		case 'plastic':
			var shininess = 150;
	        break;
		case 'wood':
			var shininess = 15;
	        break;
		case 'metal':
			var shininess = 500;
	        break;
		default:
			var shininess = 150;
	}
	jQuery('#woo3dv_model_shininess').val(model_shininess)
	woo3dv.model_mesh.material.shininess=shininess;
}

function woo3dvGetCurrentTransparency() {

	switch(woo3dv.model_transparency) {
		case 'opaque':
			var transparency = 1;
	        break;
		case 'resin':
			var transparency = 0.8;
	        break;
		case 'glass':
			var transparency = 0.6;
	        break;
		default:
			var transparency = 1;

	}
	return transparency;
}

function woo3dvSetCurrentTransparency(model_transparency) {

	switch(model_transparency) {
		case 'opaque':
			var transparency = 1;
	        break;
		case 'resin':
			var transparency = 0.8;
	        break;
		case 'glass':
			var transparency = 0.6;
	        break;
		default:
			var transparency = 1;

	}
	jQuery('#woo3dv_model_transparency').val(model_transparency)
	woo3dv.model_mesh.material.opacity=transparency;
}




function woo3dvDisplayUserDefinedProgressBar(show) {
	if(show) {
		jQuery('#woo3dv-file-loading').show();
	}
	else {
		if (!woo3dv.repairing) {
			jQuery('#woo3dv-file-loading').hide();
		}
	}
}



function woo3dvNoEnter(e) {
    if (e.which == 13) {
        return false;
    }
}

THREE.STLLoader.prototype.parseASCII = function ( data ) {
		var geometry, length, normal, patternFace, patternNormal, patternVertex, result, text;
		geometry = new THREE.BufferGeometry();
		patternFace = /facet([\s\S]*?)endfacet/g;



		var vertices = new Array();
		var normals = new Array();

		while ( ( result = patternFace.exec( data ) ) !== null ) {

			text = result[ 0 ];
			patternNormal = /normal[\s]+([\-+]?[0-9]+\.?[0-9]*([eE][\-+]?[0-9]+)?)+[\s]+([\-+]?[0-9]*\.?[0-9]+([eE][\-+]?[0-9]+)?)+[\s]+([\-+]?[0-9]*\.?[0-9]+([eE][\-+]?[0-9]+)?)+/g;

			while ( ( result = patternNormal.exec( text ) ) !== null ) {

				normal = new THREE.Vector3( parseFloat( result[ 1 ] ), parseFloat( result[ 3 ] ), parseFloat( result[ 5 ] ) );

				normals.push(result[ 1 ]);
				normals.push(result[ 3 ]);
				normals.push(result[ 5 ]);
			}


			patternVertex = /vertex[\s]+([\-+]?[0-9]+\.?[0-9]*([eE][\-+]?[0-9]+)?)+[\s]+([\-+]?[0-9]*\.?[0-9]+([eE][\-+]?[0-9]+)?)+[\s]+([\-+]?[0-9]*\.?[0-9]+([eE][\-+]?[0-9]+)?)+/g;
			tetrahedron = new Array();
			var i = 1;

			while ( ( result = patternVertex.exec( text ) ) !== null ) {

				tetrahedron[i] = new Array();
				tetrahedron[i].push(parseFloat( result[ 1 ] ));
				tetrahedron[i].push(parseFloat( result[ 3 ] ));
				tetrahedron[i].push(parseFloat( result[ 5 ] ));

				vertices.push(parseFloat(result[ 1 ]));
				vertices.push(parseFloat(result[ 3 ]));
				vertices.push(parseFloat(result[ 5 ]));

				i++;
			}



		}

		var vertices32 = new Float32Array(vertices);
		var normals32 = new Float32Array(normals);
		geometry.addAttribute( 'position', new THREE.BufferAttribute( vertices32, 3 ) );
		geometry.addAttribute( 'normal', new THREE.BufferAttribute( normals32, 3 ) );

		geometry.computeBoundingBox();
		geometry.computeBoundingSphere();

		return geometry;
}
function woo3dvRotateModel(axis, degree) {
	if (isNaN(degree)) degree=0;

	if (axis=='x') {
		jQuery("#rotation_x").focus();
		woo3dv.model_mesh.rotation.x=woo3dv.initial_rotation_x+woo3dvAngleToRadians(degree);

		if (woo3dv.object.type=='Group') {
			woo3dv.object.rotation.x=woo3dv.initial_rotation_x+woo3dvAngleToRadians(degree);
		}
	}
	if (axis=='y') {
		jQuery("#rotation_y").focus();
		woo3dv.model_mesh.rotation.y=woo3dv.initial_rotation_y+woo3dvAngleToRadians(degree);
		if (woo3dv.object.type=='Group') {
			woo3dv.object.rotation.y=woo3dv.initial_rotation_y+woo3dvAngleToRadians(degree);
		}
	}
	if (axis=='z') {
		jQuery("#rotation_z").focus();
		woo3dv.model_mesh.rotation.z=woo3dv.initial_rotation_z+woo3dvAngleToRadians(degree);
		if (woo3dv.object.type=='Group') {
			woo3dv.object.rotation.z=woo3dv.initial_rotation_z+woo3dvAngleToRadians(degree);
		}
	}
	if (woo3dv.object.type=='Group') {
		//calculate new dimensions
		var bbox = new THREE.Box3().setFromObject(woo3dv.object);
		var mesh_height = bbox.max.y - bbox.min.y;
		var mesh_width = bbox.max.x - bbox.min.x;
		var mesh_length = bbox.max.z - bbox.min.z;
		woo3dv.object.position.y = woo3dv.scene.getObjectByName('ground').position.y+(mesh_height/2)
	}
	else {
		//calculate new dimensions
		var bbox = new THREE.Box3().setFromObject(woo3dv.model_mesh);
		var mesh_height = bbox.max.y - bbox.min.y;
		var mesh_width = bbox.max.x - bbox.min.x;
		var mesh_length = bbox.max.z - bbox.min.z;
		woo3dv.model_mesh.position.y = woo3dv.scene.getObjectByName('ground').position.y+(mesh_height/2)
	}
	woo3dv.axis.position.y = woo3dv.scene.getObjectByName('ground').position.y+(mesh_height/2);
	woo3dv.spritey_z.position.y = woo3dv.scene.getObjectByName('ground').position.y+mesh_height+10;
	woo3dv.spritey_x.position.y = woo3dv.scene.getObjectByName('ground').position.y+(mesh_height/2);
	woo3dv.spritey_x.position.x = woo3dv.scene.getObjectByName('ground').position.x+(mesh_width/2)+10;
	woo3dv.spritey_y.position.y = woo3dv.scene.getObjectByName('ground').position.y+(mesh_height/2);
	woo3dv.spritey_y.position.z = woo3dv.scene.getObjectByName('ground').position.z+(mesh_length/2)+10;
}
function woo3dvAngleToRadians (angle) {
	return angle * (Math.PI / 180);
}
