<?php

/**
 *
 *
 * @author Sergey Burkov, http://www.wp3dprinting.com
 * @copyright 2017
 */

function woo3dv_activate() {
	global $wpdb;

	if ( !in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
		deactivate_plugins( plugin_basename( __FILE__ ) );
		wp_die ('Woocommerce is not installed!');
	}

	$current_version = get_option( 'woo3dv_version');


	woo3dv_check_install();

	add_option( 'woo3dv_do_activation_redirect', true );

	update_option( 'woo3dv_version', WOO3DV_VERSION );

	do_action( 'woo3dv_activate' );
}

function woo3dv_check_install() {
	global $wpdb;

	$current_version = get_option( 'woo3dv_version');
	$charset_collate = $wpdb->get_charset_collate();

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

	$default_image_url = str_replace('http:','',plugins_url()).'/woo-3d-viewer/images/';

	$current_settings = get_option( 'woo3dv_settings' );

	$settings=array(
		'file_extensions' => 'stl,obj,zip',
		'canvas_width' => (isset($current_settings['canvas_width']) ? $current_settings['canvas_width'] : '1024'),
		'canvas_height' => (isset($current_settings['canvas_height']) ? $current_settings['canvas_height'] : '768'),
		'shading' => (isset($current_settings['shading']) ? $current_settings['shading'] : 'flat'),
		'auto_rotation' => (isset($current_settings['auto_rotation']) ? $current_settings['auto_rotation'] : 'on'),
		'background1' => (isset($current_settings['background1']) ? $current_settings['background1'] : '#FFFFFF'),
		'background2' => (isset($current_settings['background2']) ? $current_settings['background2'] : '#1e73be'),
		'plane_color' => (isset($current_settings['plane_color']) ? $current_settings['plane_color'] : '#898989'),
		'ground_color' => (isset($current_settings['ground_color']) ? $current_settings['ground_color'] : '#c1c1c1'),
		'ground_mirror' => (isset($current_settings['ground_mirror']) ? $current_settings['ground_mirror'] : ''),
		'show_shadow' => (isset($current_settings['show_shadow']) ? $current_settings['show_shadow'] : ''),
		'ajax_loader' => (isset($current_settings['ajax_loader']) ? $current_settings['ajax_loader'] : $default_image_url.'ajax-loader.gif'),
		'show_grid' => (isset($current_settings['show_grid']) ? $current_settings['show_grid'] : 'on'),
		'api_login' => (isset($current_settings['api_login']) ? $current_settings['api_login'] : ''),
		'load_everywhere' => (isset($current_settings['load_everywhere']) ? $current_settings['load_everywhere'] : ''),
		'mobile_no_animation' => (isset($current_settings['mobile_no_animation']) ? $current_settings['mobile_no_animation'] : '')

	);


	update_option( 'woo3dv_settings', $settings );

	$upload_dir = wp_upload_dir();

	if ( !is_dir( $upload_dir['basedir'].'/woo3dv/' ) ) {
		mkdir( $upload_dir['basedir'].'/woo3dv/' );
	}

	if ( !file_exists( $upload_dir['basedir'].'/woo3dv/index.html' ) ) {
		$fp = fopen( $upload_dir['basedir'].'/woo3dv/index.html', "w" );
		fclose( $fp );
	}


	update_option( 'woo3dv_version', WOO3DV_VERSION );

}

function woo3dv_filter_update_checks($queryArgs) {
	$settings = get_option('woo3dv_settings');
	if ( !empty($settings['api_login']) ) {
		$queryArgs['login'] = $settings['api_login'];

	}
	return $queryArgs;
}

function woo3dv_get_option ($option) {
	return get_option($option);
}

function woo3dv_add_option ($option, $data) {

	add_option($data);
}

function woo3dv_update_option ($option, $data) {
	update_option($option, $data);
}

add_action( 'plugins_loaded', 'woo3dv_load_textdomain' );
function woo3dv_load_textdomain() {
	load_plugin_textdomain( 'woo3dv', false, dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/languages/' );
}



function woo3dv_enqueue_scripts_backend() {
	global $wp_scripts;
	$woo3dv_current_version = get_option('woo3dv_version');
	$settings = get_option( 'woo3dv_settings' );
	wp_enqueue_script( 'js/woo3dv-backend.js', plugin_dir_url( __FILE__ ).'js/woo3dv-backend.js', array( 'jquery' ), $woo3dv_current_version );

	if (isset($_GET['page']) && $_GET['page']=='woo3dv') {
		wp_enqueue_script( 'jquery' );
		wp_enqueue_script( 'jquery-ui-core' );
		wp_enqueue_script( 'jquery-ui-tabs' );
		wp_enqueue_script( 'jquery-ui-dialog' );
#		wp_enqueue_script( 'jquery.sumoselect.min.js',  plugin_dir_url( __FILE__ ).'ext/sumoselect/jquery.sumoselect.min.js', array( 'jquery' ), $woo3dv_current_version );
#		wp_enqueue_style( 'sumoselect.css', plugin_dir_url( __FILE__ ).'ext/sumoselect/sumoselect.css', array(), $woo3dv_current_version );
		wp_enqueue_script( 'tooltipster.js',  plugin_dir_url( __FILE__ ).'ext/tooltipster/js/jquery.tooltipster.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_style( 'tooltipster.css', plugin_dir_url( __FILE__ ).'ext/tooltipster/css/tooltipster.css', array(), $woo3dv_current_version );
		wp_enqueue_style( 'jquery-ui.min.css', plugin_dir_url( __FILE__ ).'ext/jquery-ui/jquery-ui.min.css', array(), $woo3dv_current_version );
		wp_enqueue_style( 'woo3dv-backend.css', plugin_dir_url( __FILE__ ).'css/woo3dv-backend.css', array(), $woo3dv_current_version );
	}

	if (isset($_GET['post']) && is_numeric($_GET['post']) && $_GET['action']=='edit' && woo3dv_is_woo3dv($_GET['post'])) {
		wp_enqueue_script( 'jquery-ui' );
		wp_enqueue_script( 'jquery-ui-dialog' );
		wp_enqueue_script( 'woo3dv-threejs',  plugin_dir_url( __FILE__ ).'ext/threejs/three.min.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-detector',  plugin_dir_url( __FILE__ ).'ext/threejs/js/Detector.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-mirror',  plugin_dir_url( __FILE__ ).'ext/threejs/js/Mirror.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-controls',  plugin_dir_url( __FILE__ ).'ext/threejs/js/controls/OrbitControls.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-canvas-renderer',  plugin_dir_url( __FILE__ ).'ext/threejs/js/renderers/CanvasRenderer.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-projector-renderer',  plugin_dir_url( __FILE__ ).'ext/threejs/js/renderers/Projector.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-stl-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/STLLoader.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-obj-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/OBJLoader.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-mtl-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/MTLLoader.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-backend-model.js',  plugin_dir_url( __FILE__ ).'js/woo3dv-backend-model.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_style( 'woo3dv-backend.css', plugin_dir_url( __FILE__ ).'css/woo3dv-backend.css', array(), $woo3dv_current_version );

		wp_localize_script( 'woo3dv-backend-model.js', 'woo3dv',
			array(
				'url' => admin_url( 'admin-ajax.php' ),
				'plugin_url' => plugin_dir_url( dirname( __FILE__ ) ),
				'shading' => $settings['shading'],
				'display_mode' => '3d_model',
				'show_shadow' => $settings['show_shadow'],
				'ground_mirror' => $settings['ground_mirror'],
				'background1' => str_replace( '#', '0x', $settings['background1']),
				'plane_color' => str_replace( '#', '0x', $settings['plane_color'] ),
				'ground_color' => str_replace( '#', '0x', $settings['ground_color'] ),
				'auto_rotation' => $settings['auto_rotation'],
				'show_grid' => $settings['show_grid'],
				'admin_preview_model' => $settings['admin_preview_model'],
				'post_max_size' => ini_get('post_max_size'),
				'text_enable_preview' => __('Please enable Preview Model in the settings of the plugin', 'woo3dv'),
				'text_upload_model' => __('Please upload the model first', 'woo3dv')

			)
		);
	}
}

function woo3dv_enqueue_scripts_frontend() {
	global $post, $woocommerce;
	if ( is_shop() ) return false;

	$available_variations = array();

	$product_model = get_post_meta( get_the_ID(), '_product_model', true );

	$woo3dv_current_version = get_option('woo3dv_version');

	$settings = get_option( 'woo3dv_settings' );

	if (woo3dv_is_woo3dv(get_the_ID())) {
		if ($settings['load_everywhere']!='on') $condition = (strlen( $product_model ) );
		else $condition = true;
	}
		else $condition = false;


	if ( $condition ) {
		wp_enqueue_style( 'woo3dv-frontend.css', plugin_dir_url( __FILE__ ).'css/woo3dv-frontend.css', array(), $woo3dv_current_version );

		wp_enqueue_style( 'tooltipster.bundle.min.css', plugin_dir_url( __FILE__ ).'ext/tooltipster/css/tooltipster.bundle.min.css', array(), $woo3dv_current_version );
		wp_enqueue_style( 'tooltipster-sideTip-light.min.css ', plugin_dir_url( __FILE__ ).'ext/tooltipster/css/plugins/tooltipster/sideTip/themes/tooltipster-sideTip-light.min.css', array(), $woo3dv_current_version );

		if( version_compare( $woocommerce->version, '3.0', ">=" ) ) {
			wp_enqueue_style( 'prettyPhoto.css', plugin_dir_url( __FILE__ ).'ext/prettyPhoto/css/prettyPhoto.css', array(), $woo3dv_current_version );
		}

		wp_enqueue_script( 'woo3dv-threejs',  plugin_dir_url( __FILE__ ).'ext/threejs/three.min.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-detector',  plugin_dir_url( __FILE__ ).'ext/threejs/js/Detector.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-mirror',  plugin_dir_url( __FILE__ ).'ext/threejs/js/Mirror.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-controls',  plugin_dir_url( __FILE__ ).'ext/threejs/js/controls/OrbitControls.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-canvas-renderer',  plugin_dir_url( __FILE__ ).'ext/threejs/js/renderers/CanvasRenderer.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-projector-renderer',  plugin_dir_url( __FILE__ ).'ext/threejs/js/renderers/Projector.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-stl-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/STLLoader.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-obj-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/OBJLoader.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-mtl-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/MTLLoader.js', array( 'jquery' ), $woo3dv_current_version );
/*		wp_enqueue_script( 'woo3dv-threejs-collada-loader-animation',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/collada/Animation.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-collada-loader-animationhandler',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/collada/AnimationHandler.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-collada-loader-animation-keyframeanimation',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/collada/KeyFrameAnimation.js', array( 'jquery' ), $woo3dv_current_version );
		wp_enqueue_script( 'woo3dv-threejs-collada-loader',  plugin_dir_url( __FILE__ ).'ext/threejs/js/loaders/ColladaLoader.js', array( 'jquery' ), $woo3dv_current_version );*/



		if( version_compare( $woocommerce->version, '3.0', ">=" ) ) {
			wp_enqueue_script( 'jquery.prettyPhoto.min.js',  plugin_dir_url( __FILE__ ).'ext/prettyPhoto/js/jquery.prettyPhoto.min.js', array( 'jquery' ), $woo3dv_current_version );
			wp_enqueue_script( 'jquery.prettyPhoto.init.min.js',  plugin_dir_url( __FILE__ ).'ext/prettyPhoto/js/jquery.prettyPhoto.init.min.js', array( 'jquery' ), $woo3dv_current_version );
		}
		wp_enqueue_script( 'woo3dv-frontend.js',  plugin_dir_url( __FILE__ ).'js/woo3dv-frontend.js', array( 'jquery' ), $woo3dv_current_version );

		$settings=get_option( 'woo3dv_settings' );

		$woo3dv_file_url = get_post_meta(get_the_ID(), 'woo3dv_file_url', true); 


		wp_localize_script( 'woo3dv-frontend.js', 'woo3dv',
			array(
				'url' => admin_url( 'admin-ajax.php' ),
				'plugin_url' => plugin_dir_url( dirname( __FILE__ ) ),
				'shading' => $settings['shading'],
				'display_mode' => '3d_model',
				'show_shadow' => $settings['show_shadow'],
				'ground_mirror' => $settings['ground_mirror'],
				'background1' => str_replace( '#', '0x', $settings['background1']),
				'plane_color' => str_replace( '#', '0x', $settings['plane_color'] ),
				'ground_color' => str_replace( '#', '0x', $settings['ground_color'] ),
				'auto_rotation' => $settings['auto_rotation'],
				'show_grid' => $settings['show_grid'],
				'mobile_no_animation' => $settings['mobile_no_animation']
			)
		);

		if  ( woo3dv_is_woo3dv ( get_the_ID() ) || $condition ) {
			$fix_css = "
				.product.has-default-attributes.has-children > .images {
					opacity:1 !important;
				}
				@media screen and (max-width: 400px) {
				   .product.has-default-attributes.has-children > .images { 
				    float: none;
				    margin-right:0;
				    width:auto;
				    border:0;
				    border-bottom:2px solid #000;    
				  }
				}
				@media screen and (max-width:800px){
					.product.has-default-attributes.has-children > .images  {
						width: auto !important;
					}

				}
			";
			wp_add_inline_style( 'woo3dv-frontend.css', $fix_css );
		}
	}


	
}

function woo3dv_handle_upload() {
	error_log(print_r($_POST, true)."\n", 3, 'C:\wamp64\www\log.log');
}

add_action( 'admin_init', 'woo3dv_plugin_redirect' );
function woo3dv_plugin_redirect() {
	if ( get_option( 'woo3dv_do_activation_redirect', false ) ) {
		delete_option( 'woo3dv_do_activation_redirect' );
		if ( !isset( $_GET['activate-multi'] ) ) {
			wp_redirect( admin_url( 'admin.php?page=woo3dv' ) );exit;
		}
	}
}

function woo3dv_extension($file) {
	$array=explode('.',$file);
	$ext=array_pop($array);
	return $ext;
} 




function woo3dv_deactivate() {
	global $wpdb;

	do_action( 'woo3dv_deactivate' );
}

function woo3dv_delete_woo3dv( $post_id ) {

}

function woo3dv_is_woo3dv( $post_id ) {
	$product_model = get_post_meta( $post_id, '_product_model', true );
	if (strlen($product_model)) return true;

#	$product_image_png = get_post_meta( $post_id, '_product_image_png', true );
#	if (strlen($product_image_png)) return true;

#	$product_image_gif = get_post_meta( $post_id, '_product_image_gif', true );
#	if (strlen($product_image_gif)) return true;

#	$product_video_webm = get_post_meta( $post_id, '_product_video_webm', true );
#	if (strlen($product_video_webm)) return true;


	return false;
}


function woo3dv_save_thumbnail( $data, $filename ) {
	$link = '';
	if ( !empty($data) ) {
//		$new_filename=$filename.'.png';
		$new_filename=$filename;
		$upload_dir = wp_upload_dir();
		$file_path=$upload_dir['basedir'].'/woo3dv/'.$new_filename;
		file_put_contents( $file_path, base64_decode( $data ) );
		$link = $upload_dir['baseurl'].'/woo3dv/'.$new_filename;
	}
	return $link;
}


add_action( 'woocommerce_process_product_meta', 'woo3dv_save_model', 20, 2 );
function woo3dv_save_model($post_id) {
		$product_model = wc_clean($_POST['product_model']);
		$product_model = str_ireplace('http:', '', $product_model);
		$product_model = str_ireplace('https:', '', $product_model);
#		$product_image_png = wc_clean($_POST['product_image_png']);
#		$product_image_gif = wc_clean($_POST['product_image_gif']);
#		$product_video_webm = wc_clean($_POST['product_video_webm']);
		$upload_dir = wp_upload_dir();

		if (strtolower(woo3dv_extension($product_model))=='zip') {

			if (class_exists('ZipArchive')) {

				$allowed_extensions_inside_archive=woo3dv_get_allowed_extensions_inside_archive();




				$time=time();
				$targetDir = dirname($upload_dir['basedir']).'/'.dirname(substr($product_model, strpos($product_model, 'uploads/'))).'/';
				$filePath = $targetDir.woo3dv_basename($product_model);
				if (!file_exists("$targetDir/tmp")) mkdir ("$targetDir/tmp");


				$zip = new ZipArchive;
				$res = $zip->open( $filePath );

				if ( $res === TRUE ) {

					for( $i = 0; $i < $zip->numFiles; $i++ ) {
						$file_to_extract = woo3dv_basename( $zip->getNameIndex($i) );
						$f2e_path_parts = pathinfo($file_to_extract);
						$f2e_extension = mb_strtolower($f2e_path_parts['extension']);

						if (!in_array($f2e_extension, $allowed_extensions_inside_archive)) continue;

						if ( in_array($f2e_extension, woo3dv_get_accepted_models()) && !in_array($f2e_extension, woo3dv_get_support_extensions_inside_archive())) {
							$file_found = true;

							$file_to_extract = rawurlencode( woo3dv_basename( $file_to_extract ) ) ;
							$wp_filename =  $time.'_'.$file_to_extract ;
							$extension = woo3dv_extension($file_to_extract);
						}
						$zip->extractTo( "$targetDir/tmp", array( $zip->getNameIndex($i) ) );
                                                $files = woo3dv_find_all_files("$targetDir/tmp");

						foreach ($files as $filename) {

							rename($filename, $targetDir.$time."_".$file_to_extract);

							if (strtolower(woo3dv_extension($filename))=='mtl') { 
								$material_file = $time."_".$file_to_extract;
								woo3dv_process_mtl($targetDir.$time."_".$file_to_extract, $time);
							}
						}

					}

					$zip->close();
					if ( !$file_found ) {
						//die( '{"jsonrpc" : "2.0", "error" : {"code": 104, "message": "'.__( 'Model file not found.', 'woo3dv' ).'"}, "id" : "id"}' );
					}
					//copy($filePath, $targetDir.$wp_filename.'.zip');
					update_post_meta( $post_id, 'woo3dv_original_file', $targetDir.$wp_filename.'.zip' );
					update_post_meta( $post_id, 'woo3dv_original_file_url', $product_model );
				}
				else {
					//die( '{"jsonrpc" : "2.0", "error" : {"code": 105, "message": "'.__( 'Could not extract the file.', 'woo3dv' ).'"}, "id" : "id"}' );
				}
			}

				
		}
		else {

		}
		if ($product_model=='') { //removed
			$old_product_model = get_post_meta( $post_id, '_product_model', true );
			if ($old_product_model) {
				$old_product_path = woo3dv_get_path_by_url($old_product_model);
				unlink($old_product_path);
			}
		}
		update_post_meta( $post_id, '_product_model', $product_model );


		if (strlen($wp_filename)>0) {
			update_post_meta( $post_id, '_product_model', dirname($product_model).'/'.$wp_filename );
		}

		if (strlen($material_file)>0) {
			update_post_meta( $post_id, '_product_mtl', dirname($product_model).'/'.$material_file );
		}
		else {
			//delete_post_meta( $post_id, '_product_mtl' );
		}


#		print_r($_POST);
		$display_mode = wc_clean($_POST['display_mode']);
#echo $display_mode;
		update_post_meta( $post_id, '_display_mode', $display_mode );
#var_dump(get_post_meta( (int)$post_id, '_display_mode', true ));
		$product_color = wc_clean($_POST['product_color']);
		update_post_meta( $post_id, '_product_color', $product_color );

		$product_shininess = wc_clean($_POST['product_shininess']);
		update_post_meta( $post_id, '_product_shininess', $product_shininess );

		$product_transparency = wc_clean($_POST['product_transparency']);
		update_post_meta( $post_id, '_product_transparency', $product_transparency );

		$rotation_x = (int)$_POST['rotation_x'];
		update_post_meta( $post_id, '_rotation_x', $rotation_x );
		$rotation_y = (int)$_POST['rotation_y'];
		update_post_meta( $post_id, '_rotation_y', $rotation_y );
		$rotation_z = (int)$_POST['rotation_z'];
		update_post_meta( $post_id, '_rotation_z', $rotation_z );

}

function woo3dv_get_path_by_url ($url) {
	$wp_upload_dir=wp_upload_dir();
	$offset = strrpos($url, 'uploads')+7;
	$path = substr($url, $offset);
	return $wp_upload_dir['basedir'].$path;
}

function woo3dv_enable_extended_upload ( $mime_types =array() ) {
	$mime_types['stl']  = 'application/octet-stream';
	$mime_types['obj']  = 'application/octet-stream';
	$mime_types['zip']  = 'application/zip';

	return $mime_types;
}
 
add_filter('upload_mimes', 'woo3dv_enable_extended_upload');

add_action( 'admin_enqueue_scripts', 'woo3dv_add_color_picker' );
function woo3dv_add_color_picker( $hook ) {
	if ( is_admin() ) {
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_script( 'custom-script-handle', plugins_url( 'js/woo3dv-backend.js', __FILE__ ), array( 'wp-color-picker' ), false, true );
	}
}

function woo3dv_upload_file($name, $index) {
	$uploadedfile = array(
		'name'     => $_FILES[$name]['name'][$index],
		'type'     => $_FILES[$name]['type'][$index],
		'tmp_name' => $_FILES[$name]['tmp_name'][$index],
		'error'    => $_FILES[$name]['error'][$index],
		'size'     => $_FILES[$name]['size'][$index]
	);

	$upload_overrides = array( 'test_form' => false );
	$movefile = wp_handle_upload( $uploadedfile, $upload_overrides );
	if (isset( $movefile['error'])) echo $movefile['error'];
	return $movefile;
}


add_filter( 'woocommerce_single_product_image_html', 'woo3dv_woocommerce_single_product_image_html', 10, 2 );
function woo3dv_woocommerce_single_product_image_html ($image_url, $post_id) {
	global $post, $woocommerce, $product;

	if (!woo3dv_is_woo3dv(get_the_ID())) return $image_url;
	$upload_dir = wp_upload_dir();
	$product_model = get_post_meta( get_the_ID(), '_product_model', true );
	$display_mode = '3d_model';
	$product_mtl = get_post_meta( get_the_ID(), '_product_mtl', true );
	$product_color = get_post_meta( get_the_ID(), '_product_color', true );
	$product_transparency = get_post_meta( get_the_ID(), '_product_transparency', true );
	$product_shininess = get_post_meta( get_the_ID(), '_product_shininess', true );

	$rotation_x = get_post_meta( get_the_ID(), '_rotation_x', true );
	$rotation_y = get_post_meta( get_the_ID(), '_rotation_y', true );
	$rotation_z = get_post_meta( get_the_ID(), '_rotation_z', true );
	$upload_url = dirname($product_model).'/';

	$settings=get_option('woo3dv_settings');

	ob_start(); 
?>


	<div id="woo3dv-viewer" >
	<?php #if ($display_mode=='' || $display_mode=='3d_model') { ?>
		<canvas id="woo3dv-cv" width="<?php echo esc_attr($settings['canvas_width']);?>" height="<?php echo esc_attr($settings['canvas_height']);?>" style="border: 1px solid;"></canvas>
		<div id="woo3dv-file-loading">
			<img alt="Loading file" src="<?php echo esc_attr($settings['ajax_loader']); ?>">
		</div>
	<?php 
#	} 
	?>


	</div>

	<input type="hidden" id="woo3dv_model_url" value="<?php if ($display_mode=='3d_model') echo esc_attr($product_model);?>">
	<input type="hidden" id="woo3dv_model_mtl" value="<?php echo esc_attr(basename($product_mtl));?>">
	<input type="hidden" id="woo3dv_upload_url" value="<?php echo esc_attr( $upload_url ); ?>" />
	<input type="hidden" id="woo3dv_model_color" value="<?php echo esc_attr($product_color);?>">
	<input type="hidden" id="woo3dv_model_shininess" value="<?php echo esc_attr($product_shininess);?>">
	<input type="hidden" id="woo3dv_model_transparency" value="<?php echo esc_attr($product_transparency);?>">
	<input type="hidden" id="woo3dv_model_rotation_x" value="<?php echo esc_attr($rotation_x);?>">
	<input type="hidden" id="woo3dv_model_rotation_y" value="<?php echo esc_attr($rotation_y);?>">
	<input type="hidden" id="woo3dv_model_rotation_z" value="<?php echo esc_attr($rotation_z);?>">

<?php

	$image_url = ob_get_contents();
	ob_end_clean();

	return $image_url;	
}




add_filter( 'wc_get_template', 'woo3dv_get_template', 10, 2 );
function woo3dv_get_template( $located, $template_name ) {
	$woo3dv_templates=array( 'single-product/product-image.php' );

	if ( woo3dv_is_woo3dv( get_queried_object_id() ) || woo3dv_is_woo3dv( get_the_ID() ) ) {
		if ( in_array( $template_name, $woo3dv_templates ) ) {
			$woo3dv_dir = woo3dv_plugin_path();
			$located = $woo3dv_dir."/woocommerce/$template_name";
		}
	}
	return $located;
}
add_filter( 'woocommerce_locate_template', 'woo3dv_woocommerce_locate_template', 10, 3 );
function woo3dv_woocommerce_locate_template( $template, $template_name, $template_path ) {
	$_template = $template;

	if ( woo3dv_is_woo3dv( get_queried_object_id() ) || woo3dv_is_woo3dv( get_the_ID() ) ) {

		if ( ! $template_path ) $template_path = $woocommerce->template_url;
		$plugin_path  = woo3dv_plugin_path() . '/woocommerce/';

		$template = locate_template(
			array(
				$template_path . $template_name,
				$template_name
			)
		);

		if ( ! $template && file_exists( $plugin_path . $template_name ) )
			$template = $plugin_path . $template_name;
	}
	else {

	}

	if ( ! $template )
		$template = $_template;

	return $template;
}

function woo3dv_plugin_path() {
	return untrailingslashit( dirname( plugin_dir_path( __FILE__ ) ) );
}



function woo3dv_basename($file) {
	$array=explode('/',$file);
	$base=array_pop($array);
	return $base;
} 


function woo3dv_find_all_files($dir) {
	$root = scandir($dir);
	foreach($root as $value) {
	if($value === '.' || $value === '..') {continue;}
		if(is_file("$dir/$value")) {$result[]="$dir/$value";continue;}
		foreach(woo3dv_find_all_files("$dir/$value") as $value) {
			$result[]=$value;
		}
	}
	return $result;
} 

function woo3dv_get_accepted_models () {
/*
	$settings = get_option('woo3dv_settings');
	$file_extensions = explode(',', $settings['file_extensions']);
	$models = array();
	foreach ($file_extensions as $extension) {
		if ($extension=='zip') continue;
		$models[]=$extension;
		
	}
	return $models;
*/
	return array('stl', 'obj');
}

function woo3dv_get_support_extensions_inside_archive() {
	return array('mtl', 'png', 'jpg', 'jpeg', 'gif', 'tga', 'bmp');
}
function woo3dv_get_allowed_extensions_inside_archive() {
	return array_merge(woo3dv_get_accepted_models(), woo3dv_get_support_extensions_inside_archive());
}


function woo3dv_process_mtl($mtl_path, $timestamp) {
	if (file_exists($mtl_path)) {
		$new_content='';
		$handle = fopen($mtl_path, "r");  
		while (($line = fgets($handle)) !== false) {
			if (substr( trim(strtolower($line)), 0, 4 ) === "map_") {
				list ($map, $file) = explode(' ', $line, 2);
				$newline = "$map $timestamp"."_".basename($file)."\n";
			} else {
				$newline = $line;
			}
			$new_content.=$newline;
		  }
		fclose($handle);
		file_put_contents($mtl_path, $new_content);
	}
}

function woo3dv_get_mtl($file_path) {
	if (file_exists($file_path)) {
		$handle = fopen($file_path, "r");  
		while (($line = fgets($handle)) !== false) {
			if (substr( trim(strtolower($line)), 0, 6 ) === "mtllib") {
				list ($mtllib, $file) = explode(' ', $line, 2);
				list ($time, $name) = explode('_', woo3dv_basename($file_path), 2);
				return $time."_".$file;
			}
		}
	}
	return '';
}



?>